from flashgeotext.lookup import LookupDataPool
from flashgeotext.lookup import MissingLookupDataError


class GeoText(LookupDataPool):
    """Extract LookupData from input text

    GeoText inherits from LookupDataPool. It iterates through
    a LookupDataPool and interfaces flashtext.KeywordProcessor
    to extract keywords from an input text. It parses the result of
    flashtext.KeywordProcessor.extract_keywords() by counting the
    occurances of a LookupData point and optionally lists the
    span info.

    Example:

        from flashgeotext.geotext import GeoText

        geotext = GeoText(use_demo_data=True)

        input_text = '''Shanghai. The Chinese Ministry of Finance in Shanghai said that China plans
                        to cut tariffs on $75 billion worth of goods that the country
                        imports from the US. Washington welcomes the decision.'''

        geotext.extract(input_text=input_text, span_info=True)
        >> {
            'cities': {
                'Shanghai': {
                    'count': 2,
                    'span_info': [(0, 8), (45, 53)]
                    },
                'Washington, D.C.': {
                    'count': 1,
                    'span_info': [(175, 185)]
                    }
                },
            'countries': {
                'China': {
                    'count': 1,
                    'span_info': [(64, 69)]
                    },
                'United States': {
                    'count': 1,
                    'span_info': [(171, 173)]
                    }
                }
            }
    """

    def __init__(self, use_demo_data: bool = True) -> None:
        """ instantiate an empty LookupDataPool, optionally/by default with demo data

        Args:
            use_demo_data (bool): optionally use demo data, defaults to True.
        """
        self.pool: dict = {}

        if use_demo_data:
            self._add_demo_data()

    def extract(self, input_text: str, span_info: bool = True) -> dict:
        """Extract LookupData from an input_text

        Arguments:
            input_text (str): String to extract LookupData from.
            span_info (bool): Optionally, return span_info. Defaults to True.

        Returns:
            extract_ouput (dict): dictionary of extracted LookupData entities with count
                                  and optionally with listed span info.

        """
        if not self.pool:
            raise MissingLookupDataError(
                "Empty LookupDataPool. use .add(LookupData) to add data."
            )

        output: dict = {}

        for lookup in self.pool.keys():
            extract = self.pool[lookup].extract_keywords(
                input_text, span_info=span_info
            )
            output[lookup] = self._parse_extract(extract, span_info=span_info)

        return output

    def _parse_extract(self, extract_data: list, span_info: bool = True) -> dict:
        """Parse flashtext.KeywordProcessor.extract_keywords() output to count occurances

        Parse flashtext.KeywordProcessor.extract_keywords() output to count occurances,
        and optionally span_info.

        Arguments:
            extract_data (list): flashtext.KeywordProcessor.extract_keywords() return value
            span_info (bool): optionally, parse span_info

        Returns:
            parsed_extract (dict): parsed extract_data to include count, optionally span_info
        """
        parsed_extract: dict = {}

        if span_info:
            for entry in extract_data:
                if entry[0] not in parsed_extract:
                    parsed_extract[entry[0]] = {
                        "count": 1,
                        "span_info": [(entry[1], entry[2])],
                    }
                else:
                    parsed_extract[entry[0]]["count"] = (
                        parsed_extract[entry[0]]["count"] + 1
                    )
                    parsed_extract[entry[0]]["span_info"] = parsed_extract[entry[0]][
                        "span_info"
                    ] + [(entry[1], entry[2])]

        else:
            for entry in extract_data:
                if entry not in parsed_extract:
                    parsed_extract[entry] = {"count": 1}
                else:
                    parsed_extract[entry]["count"] = parsed_extract[entry]["count"] + 1

        return parsed_extract



import pytest

from flashgeotext.geotext import GeoText


@pytest.fixture
def test_data_cities():
    return {"Berlin": ["Berlin", "Dickes B"], "Hamburg": ["Hamburg", "Dickes H"]}


@pytest.fixture
def test_data_countries():
    return {
        "Germany": ["Germany", "Deutschland"],
        "Netherlands": ["The Netherlands", "Netherlands", "Holland"],
    }


@pytest.fixture
def geotext():
    return GeoText(use_demo_data=True)



import pytest

from flashgeotext.lookup import load_data_from_file
from flashgeotext.lookup import LookupData
from flashgeotext.lookup import LookupDataPool
from flashgeotext.lookup import LookupDuplicateError
from flashgeotext.settings import DEMODATA_CITIES


def test_lookup_data_pool(test_data_cities):
    lookup = LookupData(name="cities", data=test_data_cities)

    processor = LookupDataPool()
    processor.add(lookup)

    assert processor.pool[lookup.name]


def test_lookup_data_pool_duplicate_data(test_data_cities):
    lookup = LookupData(name="cities", data=test_data_cities)

    processor = LookupDataPool()
    processor.add(lookup)

    with pytest.raises(LookupDuplicateError):
        processor.add(lookup)


def test_lookup_data_pool_add_invalid_lookup(test_data_cities):
    lookup = {"test": ["test", "test2"]}

    processor = LookupDataPool()

    with pytest.raises(TypeError):
        processor.add(lookup)


def test_lookup_data_pool_duplicate_data_update_true(test_data_cities):
    lookup = LookupData(name="cities", data=test_data_cities)

    processor = LookupDataPool()
    processor.add(lookup=lookup, update=True)
    processor.add(lookup=lookup, update=True)

    assert processor.pool["cities"]


def test_lookup_data_pool_remove_lookup_from_pool(test_data_cities):
    lookup = LookupData(name="cities", data=test_data_cities)

    processor = LookupDataPool()
    processor.add(lookup)

    processor.remove(lookup_to_remove="cities")

    with pytest.raises(KeyError):
        assert processor.pool["cities"]


def test_lookup_data_pool_remove_all_from_pool(test_data_cities):
    lookup = LookupData(name="cities", data=test_data_cities)

    processor = LookupDataPool()
    processor.add(lookup)

    processor.remove_all()

    assert not processor.pool


def test_load_data_from_file():
    cities = load_data_from_file(DEMODATA_CITIES)
    assert cities


def test_load_data_from_file_raises():
    with pytest.raises(TypeError):
        cities = load_data_from_file("demodata_cities.txt")
        assert cities



import json

from flashtext import KeywordProcessor
from loguru import logger
from pydantic import BaseModel
from pydantic import StrictStr

from flashgeotext.settings import DEMODATA_CITIES
from flashgeotext.settings import DEMODATA_COUNTRIES


class LookupDuplicateError(Exception):
    """Exception is raised if LookupData is already in LookupDataPool

    Args:
        message (str): Human readable string describing the exception.

    Attributes:
        message (str): Human readable string describing the exception.
    """

    def __init__(self, message: str):
        self.message = message


class MissingLookupDataError(Exception):
    """Exception is raised if GeoText.extract() is used on empty LookupDataPool

    Args:
        message (str): Human readable string describing the exception.

    Attributes:
        message (str): Human readable string describing the exception.
    """

    def __init__(self, message: str):
        self.message = message


class LookupValidation:
    """Data validation container object

    Args:
        status (str): Humanreadible string containing the Error status.
        error_count (int): Error count in validation data.
        errors (dict):

        Example: {
            "Berlin": [
                "Berlin missing in list of synonyms",
                "data['Berlin'] is not a list of synonyms"
                ]
            }

    Arguments:
        status (str): Humanreadible string containing the Error status.
        error_count (int): Error count in validation data.
        errors (dict):

        Example: {
            "Berlin": [
                "Berlin missing in list of synonyms",
                "data['Berlin'] is not a list of synonyms"
                ]
            }
    """

    def __init__(
        self,
        status: str = "No errors detected",
        error_count: int = 0,
        errors: dict = {},
    ):
        self.status = status
        self.error_count = error_count
        self.errors = {}

    def __repr__(self):
        return f"<LookupValidation: {self.__dict__}>"


class LookupData(BaseModel, object):
    """Data that is supposed to be looked up in a text

    Args:
        name (pydantic.StrictStr): Human readable name as string describing the data.
        data (dict): dictionary containing data to lookup and their synonyms

    Attributes:
        name (pydantic.StrictStr): Human readable name as string describing the data.
        data (dict): dictionary containing data to lookup and their synonyms
    """

    name: StrictStr
    data: dict

    def validate(self) -> dict:
        """Validate if data attribute has appropiate structure.

        returns:
            LookupValidation
        """

        validation = LookupValidation()

        for key, value in self.data.items():
            if not isinstance(value, list):
                validation.errors[key] = [f"data[{key}] is not a list of synonyms"]
                validation.error_count = validation.error_count + 1

            if key not in value:
                if key in validation.errors:
                    validation.errors[key] = validation.errors[key] + [
                        f"{key} missing in list of synonyms"
                    ]
                else:
                    validation.errors[key] = [f"{key} missing in list of synonyms"]

                validation.error_count = validation.error_count + 1

        if validation.error_count > 0:
            validation.status = f"Found {validation.error_count} errors"

        return validation.__dict__


class LookupDataPool:
    """Collection of KeywordProcessors from LookupData

    Args:
        pool (dict): Collection of LookupData.

    Attributes:
        pool (dict): Collection of LookupData.

    Example:
        pool = {
            LookupData.name: flashtext.KeywordProcessor.add_keywords_from_dict(LookupData.data)
            }
    """

    def __init__(self) -> None:
        self.pool: dict = {}

    def add(self, lookup: LookupData, update: bool = False) -> None:
        """Add LookupData to LookupDataPool

        Add LookupData to LookupDataPool.
        Raises flashgeotext.lookup.LookupDuplicateError if lookup
        is already in pool unless update == True.

        Args:
            lookup (LookupData): LookupData to add to pool
            update (bool): Allow update of an existing entry in LookupDataPool
        """
        if not isinstance(lookup, LookupData):
            raise TypeError(f"lookup has to be instance of LookupData")

        if lookup.name in self.pool and not update:
            raise LookupDuplicateError(
                f"'{lookup.name}' has already been added. Set update=True to update"
            )
        else:
            self.pool[lookup.name] = KeywordProcessor(case_sensitive=True)
            self.pool[lookup.name].add_keywords_from_dict(lookup.data)
            logger.debug(f"{lookup.name} added to pool")

    def remove(self, lookup_to_remove: str) -> None:
        """Remove LookupData from LookupDataPool

        Args:
            lookup_to_remove (str): LookupData to remove from pool
        """
        if lookup_to_remove in self.pool:
            del self.pool[lookup_to_remove]
            logger.debug(f"{lookup_to_remove} removed from pool")

    def remove_all(self):
        """Remove all LookupData from LookupDataPool
        """

        self.pool = {}

    def _add_demo_data(self):
        """(private) Add demo data to pool

        Adds DEMODATA_CITIES and DEMODATA_COUNTRIES to LookupDataPool
        """
        cities = LookupData(
            name="cities", data=load_data_from_file(file=DEMODATA_CITIES)
        )
        countries = LookupData(
            name="countries", data=load_data_from_file(file=DEMODATA_COUNTRIES)
        )
        self.add(cities)
        self.add(countries)
        logger.debug(f"demo data loaded for: {list(self.pool.keys())}")


def load_data_from_file(file: str) -> dict:
    """Load data from json file

    Load data from json file. Raises TypeError if not json

    Args:
        file (str): path to file
    """

    if not file.endswith(".json"):
        raise TypeError("File has to be Filetype .json")

    with open(file, "r", encoding="utf-8") as f:
        return json.loads(f.read())



import os

PARENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))
DEMODATA_DIR = os.path.join(PARENT_DIRECTORY, "resources")
DEMODATA_CITIES = DEMODATA_DIR + "/cities.json"
DEMODATA_COUNTRIES = DEMODATA_DIR + "/countries.json"



import pytest

from flashgeotext.geotext import GeoText
from flashgeotext.lookup import MissingLookupDataError

text = "Berlin ist die Hauptstadt von Deutschland. Berlin ist nicht haesslich, aber auch nicht sonderlich schoen."


def test_geotext_demo_data():
    geotext = GeoText(use_demo_data=True)

    assert geotext.pool["cities"]
    assert geotext.pool["countries"]


def test_geotext_extract(geotext):
    output = geotext.extract(text)
    assert "Berlin" in output["cities"]


def test_geotext_raises_on_empty_pool():
    output = GeoText(use_demo_data=False)

    with pytest.raises(MissingLookupDataError):
        output.extract(text)


def test_geotext_extract_with_count_span_info_true(geotext):
    output = geotext.extract(input_text=text, span_info=True)
    assert output["cities"]["Berlin"]["count"] == 2
    assert output["cities"]["Berlin"]["span_info"] == [(0, 6), (43, 49)]


def test_geotext_extract_with_count_span_info_false(geotext):
    output = geotext.extract(input_text=text, span_info=False)

    with pytest.raises(KeyError):
        assert output["cities"]["Berlin"]["span_info"] == [(0, 6), (43, 49)]


# tests used in geotext (https://github.com/elyase/geotext)


@pytest.mark.parametrize(
    "nr, text, expected_cities",
    [
        pytest.param(
            1,
            """As capitais do nordeste brasileiro são:
            Salvador na Bahia, Recife em Pernambuco, Natal fica no Rio Grande do Norte, João Pessoa fica na Paraíba,
            Fortaleza fica no Ceará, Teresina no Piauí, Aracaju em Sergipe, Maceió em Alagoas e São Luís no Maranhão.""",
            [
                "Salvador",
                "Recife",
                "Natal",
                "Rio Grande",
                "João Pessoa",
                "Fortaleza",
                "Teresina",
                "Aracaju",
                "Maceió",
                "São Luís",
            ],
            id="northeast brazil",
        ),
        pytest.param(
            2,
            """As capitais dos estados do norte brasileiro são: Manaus no Amazonas,
            Palmas em Tocantins, Belém no Pará, Acre no Rio Branco.""",
            ["Manaus", "Palmas", "Belém", "Rio Branco"],
            id="northern brazil",
        ),
        pytest.param(
            3,
            """São Paulo é a capital do estado de São Paulo. As cidades de Barueri
            e Carapicuíba fazem parte da Grade São Paulo. O Rio de Janeiro continua lindo.
            No carnaval eu vou para Salvador. No reveillon eu quero ir para Santos.""",
            [
                "São Paulo",
                "São Paulo",
                "Barueri",
                "Carapicuíba",
                "Rio de Janeiro",
                "Salvador",
                "Santos",
            ],
            id="southeast brazil",
        ),
        pytest.param(
            4,
            """As capitais da região centro-oeste do Brasil são:
            Goiânia em Goiás, Brasília no Distrito Federal, Campo Grande no Mato Grosso do Sul,
            Cuiabá no Mato Grosso.""",
            ["Goiânia", "Goiás", "Brasília", "Campo Grande", "Cuiabá"],
            id="central brazil",
        ),
        pytest.param(
            5,
            """As capitais da região sul são:
            Porto Alegre no Rio Grande do Sul, Floripa em Santa Catarina,
            Curitiba no Paraná""",
            ["Porto Alegre", "Rio Grande", "Santa Catarina", "Curitiba", "Paraná"],
            id="south brazil",
        ),
        pytest.param(
            6,
            """A high-speed passenger train derailed in the early hours of Thursday morning near Milan, northern Italy.
            Officials said that the train was traveling at 180 mph when the engine car derailed,
            separated from the train, and slammed into a railroad building. The train was travelling to
            Bologna, with 33 people on board. Two railway workers were killed, and 27 people were injured.""",
            ["Milano", "Bologna"],
            id="english1-synonym",
        ),
    ],
)
def test_geotext_extract_cities(nr, text, expected_cities, geotext):
    output = geotext.extract(input_text=text, span_info=False)

    assert all([city in output["cities"] for city in expected_cities])


@pytest.mark.parametrize(
    "nr, text, expected_countries",
    [
        pytest.param(
            1,
            """The total use of coal to generate electricity is set
            to decrease by 3 percent this year. This amounts to the
            total coal use by Germany, UK, and Spain combined.""",
            ["Germany", "United Kingdom", "Spain"],
            id="english1-synonym",
        ),
        pytest.param(
            2,
            """The Chinese Ministry of Finance said that China plans
            to cut tariffs on $75 billion worth of goods that the country
            imports from the US.""",
            ["China", "United States"],
            id="english2-synonym",
        ),
        pytest.param(
            2,
            """Israel wants to take a part of Palestine’s area. Palestine is not happy about it.
            It is not good for the power of Palestine. People in Palestine are angry.
            Palestine is inside Israel if Israel takes the area.
            The US makes a new plan for peace for Israel and Palestine.
            The US agrees with Israel. The US supports Israel´s plan to take a part of Palestine.""",
            ["Palestine", "United States", "Israel"],
            id="english3-synonym",
        ),
    ],
)
def test_geotext_extract_countries(nr, text, expected_countries, geotext):
    output = geotext.extract(input_text=text, span_info=False)

    assert all([country in output["countries"] for country in expected_countries])



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['flashgeotext']

package_data = \
{'': ['*'], 'flashgeotext': ['resources/*', 'resources/.ipynb_checkpoints/*']}

install_requires = \
['flashtext==2.7', 'loguru==0.4.1', 'pydantic==1.4']

extras_require = \
{'doc': ['awscli',
         'mkdocs',
         'mkdocs-material',
         'markdown-include',
         'pydoc-markdown'],
 'test': ['pytest >=4.0.0', 'pytest-cov']}

setup(name='flashgeotext',
      version='0.1.0',
      description='Extract and count countries and cities (+their synonyms) from text',
      author='Benjamin Ramser',
      author_email='ahoi@iwpnd.pw',
      url='https://github.com/iwpnd/flashgeotext',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )



import pytest
from pydantic import ValidationError

from flashgeotext.lookup import load_data_from_file
from flashgeotext.lookup import LookupData
from flashgeotext.lookup import LookupValidation
from flashgeotext.settings import DEMODATA_CITIES
from flashgeotext.settings import DEMODATA_COUNTRIES


def test_lookup_data(test_data_cities):
    lookup = LookupData(name="cities", data=test_data_cities)

    assert lookup.name == "cities"
    assert isinstance(lookup.data, dict)


@pytest.mark.parametrize(
    "id, name, data, expectation",
    [
        (1, "cities", ["Berlin", "Hamburg"], pytest.raises(ValidationError)),
        (2, "cities", ("Berlin", "Hamburg"), pytest.raises(ValidationError)),
        (3, "cities", "Berlin", pytest.raises(ValidationError)),
        (4, "cities", 1337, pytest.raises(ValidationError)),
        (5, 1337, {"Berlin": ["Berlin"]}, pytest.raises(ValidationError)),
        (6, [13, 37], {"Berlin": ["Berlin"]}, pytest.raises(ValidationError)),
        (7, (13, 37), {"Berlin": ["Berlin"]}, pytest.raises(ValidationError)),
        (8, {"13": 37}, {"Berlin": ["Berlin"]}, pytest.raises(ValidationError)),
    ],
)
def test_lookup_data_raises(id, name, data, expectation):

    with expectation:
        lookup = LookupData(name=name, data=data)

        assert isinstance(lookup, LookupData)


@pytest.mark.parametrize(
    "id, name, data, error_count",
    [
        (1, "cities", {"Berlin": "Berlin"}, 1),
        (2, "cities", {"Berlin": ["Dickes B"]}, 1),
        (2, "cities", {"Berlin": "Hamburg"}, 2),
    ],
)
def test_lookup_data_validate(id, name, data, error_count):
    lookup = LookupData(name=name, data=data)

    validation = lookup.validate()

    assert validation["error_count"] == error_count


@pytest.mark.parametrize(
    "id, name, demodata",
    [
        pytest.param(1, "cities", DEMODATA_CITIES, id="demodata_cities"),
        pytest.param(2, "countries", DEMODATA_COUNTRIES, id="demodata_countries"),
    ],
)
def test_lookup_data_demo_data(id, name, demodata):
    lookup = LookupData(name=name, data=load_data_from_file(demodata))

    validation = lookup.validate()

    assert validation["error_count"] == 0


def test_lookup_data_validate_repr():
    lookup = LookupData(name="countries", data=load_data_from_file(DEMODATA_COUNTRIES))

    assert isinstance(lookup.validate(), dict)


def test_lookupvalidation_repr():
    validation = LookupValidation()

    assert (
        repr(validation)
        == "<LookupValidation: {'status': 'No errors detected', 'error_count': 0, 'errors': {}}>"
    )
